unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, clExWatcher, ShowExcept;

type
  TMainForm = class(TForm)
    btnDelphiException: TButton;
    btnDelphiSafeCallException: TButton;
    btnSystemException: TButton;
    clExceptWatcher: TclExceptWatcher;
    btnSystemSafeCallException: TButton;
    btnTryExcept: TButton;
    memStackLog: TMemo;
    procedure clExceptWatcherDelphiException(Sender: TObject; E: Exception;
      EI: TclExceptionInformation; StackTracer: TclStackTracer);
    procedure clExceptWatcherDelphiSafeCallException(Sender,
      Target: TObject; E: Exception; EI: TclExceptionInformation;
      StackTracer: TclStackTracer);
    procedure clExceptWatcherSystemException(Sender: TObject;
      EI: TclExceptionInformation; StackTracer: TclStackTracer);
    procedure clExceptWatcherSystemSafeCallException(Sender,
      Target: TObject; EI: TclExceptionInformation;
      var NotifyTarget: Boolean; StackTracer: TclStackTracer);
    procedure btnDelphiExceptionClick(Sender: TObject);
    procedure btnDelphiSafeCallExceptionClick(Sender: TObject);
    procedure btnSystemExceptionClick(Sender: TObject);
    procedure btnSystemSafeCallExceptionClick(Sender: TObject);
    procedure btnTryExceptClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure SimulateDelphiException();
    procedure SimulateSystemException();
    procedure SimulateDelphiSafeCallException(); safecall;
    procedure SimulateSystemSafeCallException(); safecall;
    procedure ShowStack(Frames: TclStackFrames);
    procedure ExceptionHandler(Sender: TObject; E: Exception);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.clExceptWatcherDelphiException(Sender: TObject;
  E: Exception; EI: TclExceptionInformation; StackTracer: TclStackTracer);
begin
  memStackLog.Clear();
  memStackLog.Lines.Add('Delphi Exception:');
  ShowStack(StackTracer.GetCallStack(EI));
end;

procedure TMainForm.clExceptWatcherSystemException(Sender: TObject;
  EI: TclExceptionInformation; StackTracer: TclStackTracer);
begin
  memStackLog.Clear();
  memStackLog.Lines.Add('System Exception:');
  ShowStack(StackTracer.GetCallStack(EI));
end;

procedure TMainForm.clExceptWatcherDelphiSafeCallException(Sender,
  Target: TObject; E: Exception; EI: TclExceptionInformation;
  StackTracer: TclStackTracer);
begin
  memStackLog.Clear();
  memStackLog.Lines.Add('Delphi SafeCall Exception:');
  ShowStack(StackTracer.GetCallStack(EI));
end;

procedure TMainForm.clExceptWatcherSystemSafeCallException(Sender,
  Target: TObject; EI: TclExceptionInformation; var NotifyTarget: Boolean;
  StackTracer: TclStackTracer);
begin
  memStackLog.Clear();
  memStackLog.Lines.Add('System SafeCall Exception:');
  ShowStack(StackTracer.GetCallStack(EI));
end;

procedure TMainForm.btnDelphiExceptionClick(Sender: TObject);
begin
  SimulateDelphiException();
end;

procedure TMainForm.btnSystemExceptionClick(Sender: TObject);
begin
  SimulateSystemException();
end;

procedure TMainForm.btnDelphiSafeCallExceptionClick(Sender: TObject);
begin
  SimulateDelphiSafeCallException();
end;

procedure TMainForm.btnSystemSafeCallExceptionClick(Sender: TObject);
begin
  SimulateSystemSafeCallException();
end;

procedure TMainForm.btnTryExceptClick(Sender: TObject);
begin
  try
    try
      SimulateSystemException();
    except
      raise;
    end;
  except
  end;
end;

procedure TMainForm.SimulateDelphiException;
begin
  raise Exception.Create('Test Delphi exception');
end;

procedure TMainForm.SimulateSystemException;
var
  p: PChar;
begin
  p := PChar(5);
  p^ := #9; //Access Violation here
end;

procedure TMainForm.SimulateDelphiSafeCallException;
begin
  raise Exception.Create('Test Delphi SafeCall exception');
end;

procedure TMainForm.SimulateSystemSafeCallException;
var
  p: PChar;
begin
  p := PChar(5);
  p^ := #9; //Access Violation here
end;

procedure TMainForm.ShowStack(Frames: TclStackFrames);
var
  i: Integer;
begin
  memStackLog.Lines.BeginUpdate();
  for i := 0 to Frames.Count - 1 do
  begin
    memStackLog.Lines.Add(Format('%x: %s - %s', [Integer(Frames[i].CallAddress),
      ExtractFileName(Frames[i].ModuleName), Frames[i].FunctionName]));
  end;
  memStackLog.Lines.EndUpdate();
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnException := ExceptionHandler;
end;

procedure TMainForm.ExceptionHandler(Sender: TObject; E: Exception);
begin
  ShowExceptionForm(E.Message);
end;

end.
